from __future__ import annotations

import asyncio
import importlib
import os

import discord
from discord.ext import commands

from .bot import NightlyBot


async def _main():
    token = os.getenv("DISCORD_TOKEN")
    if token is None:
        print(
            """\
DISCORD_TOKEN environment variable is not set
edit .secrets/nightly.env with the following content (substitute <token here> with the token):
DISCORD_TOKEN = <token here>\
"""
        )
        exit(1)
    bot = NightlyBot()
    await bot.login(token)
    yield bot
    await bot.load_extension("nightly.night")
    await bot.connect()
    yield bot


async def aclose(client: discord.Client):
    if not client.is_closed():
        await client.change_presence(status=discord.Status.offline)
        await client.close()


def close(client: discord.Client, loop: asyncio.AbstractEventLoop):
    loop.run_until_complete(aclose(client))


def main():
    loop = asyncio.new_event_loop()
    gen = _main()
    bot: commands.Bot = loop.run_until_complete(anext(gen))
    discord.utils.setup_logging()

    async def complete():
        return await anext(gen)

    task = loop.create_task(complete())
    try:
        loop.run_until_complete(task)
    except (KeyboardInterrupt, InterruptedError, RuntimeError):
        try:
            close(bot, loop)
        finally:
            loop.run_until_complete(task)


if __name__ == "__main__":
    main()
